
use std::{collections::VecDeque, time::SystemTime};

use egui::{epaint::Shadow, load::TexturePoll, Align2, Color32, Frame, Image, Margin, Pos2, Rect, RichText, Rounding, Stroke, Style, Vec2};

use super::{CompGetter, CompUi};

#[derive(Debug, Default, PartialEq, Clone, Copy)]
pub enum ToastType {
    #[default]
    Info,
    Warn,
    Error,
}


#[derive(Debug, Default)]
pub struct Toast {
    queue: VecDeque<(String, String, ToastType, SystemTime)>
}

component_register!(Toast);

impl CompUi for Toast {
    fn draw(ui: &mut egui::Ui, _: &mut crate::GuiState) -> crate::Result<()> {
        let screen_size = ui.ctx().screen_rect().size();
        let (w, h) = (300.0, 100.0);
        let theme = &xmpd_settings::Settings::get()?.theme;
        let mut toastw = Toast::get()?;
        let mut height_iter = 6.0;
        let mut to_remove = Vec::new();

        for (i, (title, description, toast_type, shown_since)) in toastw.queue.iter().enumerate() {
            let area = egui::Area::new(egui::Id::new(format!("toast_{i}")))
                .fixed_pos(Pos2::new(screen_size.x - w, height_iter))
                .pivot(Align2::LEFT_TOP)
                .show(ui.ctx(), |ui| {
                    ui.set_width(w);

                    let img;
                    let color;
                    match toast_type {
                        ToastType::Info => {
                            color = theme.accent_color;
                            img = Image::new(crate::data::INFO_ICON)
                                .max_size(Vec2::new(16.0, 16.0))
                                .tint(color);
                        }
                        ToastType::Warn => {
                            color = crate::data::C_WARN;
                            img = Image::new(crate::data::WARN_ICON)
                                .max_size(Vec2::new(16.0, 16.0))
                                .tint(color);
                        }
                        ToastType::Error => {
                            color = Color32::LIGHT_RED;
                            img = Image::new(crate::data::ERROR_ICON)
                                .max_size(Vec2::new(16.0, 16.0))
                                .tint(color);
                        }
                    }
                    Frame::none()
                        .stroke(Stroke::new(1.0, color))
                        .fill(theme.primary_bg_color)
                        .rounding(Rounding::same(3.0))
                        .inner_margin(Margin::same(3.0))
                        .show(ui, |ui| {
                            ui.set_width(w-9.0);
                            ui.style_mut().visuals.override_text_color = Some(theme.text_color);
                            ui.horizontal(|ui| {
                                
                                ui.add(img);
                                ui.label(RichText::new(title));
                            });
                            ui.label(
                                RichText::new(description)
                                .size(10.0)
                            );
                            ui.shrink_height_to_current();
                            // height_iter += ui.available_height();
                        }
                    )
                }
            );
            height_iter += area.response.rect.height() + 6.0;
            
            // if shown for longer than 5 seconds remove it
            if SystemTime::now().duration_since(*shown_since)?.as_secs() > 5 {
                to_remove.push(i);
            }
        }
            
        for idx in to_remove {
            toastw.queue.remove(idx);
        }

        Ok(())
    }
}


impl Toast {
    pub fn show_toast<S>(&mut self, title: S, description: S, toast_type: ToastType)
        where S: ToString 
    {
        self.queue.push_front((title.to_string(), description.to_string(), toast_type, SystemTime::now()));
    }
}

