use std::{path::PathBuf, sync::{Arc, Mutex, MutexGuard}};
use serde::{Deserialize, Serialize};
use theme::Theme;
use tooling::Tooling;

pub mod theme;
pub mod tooling;

lazy_static::lazy_static!(
    static ref SETTINGS: Arc<Mutex<Settings>> = Arc::new(Mutex::new(Settings::default()));
);

pub type Result<T> = anyhow::Result<T>;

#[derive(Debug, Clone, Serialize, Deserialize, Default)]
pub struct Settings {
    #[serde(skip)]
    settings_path: PathBuf,
    #[serde(default)]
    pub theme: Theme,
    #[serde(default)]
    pub tooling: Tooling,
}

impl Settings {
    pub fn get() -> crate::Result<MutexGuard<'static, Self>> {
        match SETTINGS.lock() {
            Ok(l) => Ok(l),
            Err(e) => Err(anyhow::anyhow!(format!("{e:?}"))),
        }
    }
    pub fn load(&mut self, path: Option<PathBuf>) -> Result<()> {
        let path = path.unwrap_or(self.settings_path.clone());
        if !path.exists() {
            let mut dir = path.clone();
            dir.pop();
            if !dir.exists() {
                std::fs::create_dir_all(dir)?;
            }
            std::fs::write(&path, "[theme]")?;
            self.save(Some(path.clone()))?;
        }
        let data = std::fs::read_to_string(&path)?;
        let data: Self = toml::from_str(&data)?;
        *self = data;
        self.settings_path = path;
        Ok(())
    }

    pub fn save(&mut self, path: Option<PathBuf>) -> Result<()> {
        let path = path.unwrap_or(self.settings_path.clone());
        let data = toml::to_string_pretty(&self)?;
        std::fs::write(&path, data)?;
        self.settings_path = path;
        Ok(())
    }
}


